/*
 Standalone Temperature Logger for ATtiny85
 Version: 2.0 Beta 1
 Author: Alex from Inside Gadgets (http://www.insidegadgets.com)
 Created: 9/01/2011
 Last Modified: 13/05/2011
 
 Used to monitor temperature every x minutes/hours which is saved 
 to the built in EEPROM. 
 
 */

//                  +-\/-+
// Ain0 (D 5) PB5  1|    |8  Vcc
// Ain3 (D 3) PB3  2|    |7  PB2 (D 2) Ain1
// Ain2 (D 4) PB4  3|    |6  PB1 (D 1) pwm1
//            GND  4|    |5  PB0 (D 0) pwm0
//                  +----+

int thermistorPin = 1; // Analog input 1
int clockPin = 3; // Analog input 3
#define ledPin PB0 // Output
#define buttonPin PB4 // Input
#define dataPin PB1 // Output

#define F_CPU 1000000 // 1 MHz
#include <avr/io.h>
#include <avr/wdt.h>
#include <math.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include "setup.c"

volatile boolean f_pin = 0; // Pin change variable

int functionSelect = 0;
int dataCount = 1;
int delayTime = -1;

int main(void) {
	
	setup();
	
	DDRB |= ((1<<ledPin) | (1<<dataPin)); // Set Outputs
	DDRB &= ~(1<<buttonPin); // Set Inputs
	
	// Pin interrupt setup
	sbi(GIMSK,PCIE); // Enable pin change interrupt
	sbi(PCMSK,PCINT4); // Apply to pin D4 (button)	
	
	while(1) {
		// Wait for input
		if (functionSelect == 0) {
			
			// Read delay time
			if (delayTime == -1) {
				delayTime = eeprom_read_byte((uint8_t*)0);
				if (delayTime > 10) { // Not a normal value
					delayTime = 0;
				}
			}
			
			// Always reset variables just in case
			PORTB &= ~((1<<ledPin) | (1<<dataPin));
			
			system_sleep();
			
			// First test we are connected to the Arduino
			int clockState = analogRead(clockPin);
			if (clockState >= 160) {
				int clockhighCount = 0;
				// Delay for 10 x 25ms = 250ms, check CLK is always high
				for (int x = 0; x < 10; x++) { 
					_delay_ms(25);
					clockState = analogRead(clockPin);
					if (clockState >= 160) {
						clockhighCount++;
					}
					else {
						clockhighCount = 0;
					}
				}
				// CLK is high, now give the 250ms HIGH as an ok
				if (clockhighCount >= 10) {
					functionSelect = 3;
					PORTB |= (1<<dataPin); // ON
					_delay_ms(250);
					PORTB &= ~(1<<dataPin); // OFF
				}
			}
			else {
				// Button was pressed
				int buttonState = 0; 
				int buttonCount = 0;
				int buttonPressed = 0;
				long resetTimer = millis();
				while (millis() < resetTimer + 1500) {
					buttonState = PINB & (1<<buttonPin);
					if (buttonState) {
						buttonCount++;
						while (buttonState && (millis() < resetTimer + 1500)) {
							buttonState = PINB & (1<<buttonPin);
							buttonPressed++;
							_delay_ms(50);
						}
					}
				}
				
				if (buttonCount == 3) { // Start logging
					functionSelect = 1;
					dataCount = 1;
					for (int x = 0; x < 3; x++) {
						PORTB |= (1<<ledPin); // ON
						_delay_ms(500);
						PORTB &= ~(1<<ledPin); // OFF
						_delay_ms(500);
					}
				}
				else if (buttonPressed >= 20) { // Change delay time
					functionSelect = 2;
					delayTime = -1;
					PORTB |= (1<<ledPin); // ON
					_delay_ms(2000);
					PORTB &= ~(1<<ledPin); // OFF
				}
			}
		}
		
		// Log Temperature
		if (functionSelect == 1) {
			PORTB |= (1<<dataPin); // ON
			int tempValue = (int) Thermistor(analogRead(thermistorPin));
			PORTB &= ~(1<<dataPin); // OFF
			
			// Calculations to store -40C to 125C int to a single byte
			byte tempStore = tempValue + 130;
			if (tempValue <= -41) { // Temp is under min temp accepted
				tempStore = 90;
			}
			if (tempValue >= 126) { // Temp is over max temp accepted
				tempStore = 255;
			}
			
			// Write to EEPROM
			if (dataCount < 512) {
				eeprom_write_byte((uint8_t*)dataCount, tempStore);
				dataCount++;
				if (dataCount < 512) {
					eeprom_write_byte((uint8_t*)dataCount, 0); // So we know where to stop when extracting data
				}
			}
			else {
				functionSelect = 0;
			}
			
			// Calculate the delay time
			int waitCounter = 0;
			int waitTime = 0;
			f_pin = 0;
			if (delayTime == 0) {
				waitTime = 15;
			}
			if (delayTime >= 1 && delayTime < 4) {
				waitTime = (delayTime * 5) * 15;
			}
			if (delayTime >= 4 && delayTime < 6) {
				waitTime = ((delayTime-3) * 30) * 15;
			}
			if (delayTime >= 6 && delayTime < 8) {
				waitTime = ((delayTime-5) * 120) * 15;
			}
			if (delayTime == 8) {
				waitTime = 8 * 60 * 15;
			}
			if (delayTime == 9) {
				waitTime = 12 * 60 * 15;
			}
			if (delayTime == 10) {
				waitTime = 24 * 60 * 15;
			}
			
			while (waitCounter != waitTime) {
				setup_watchdog(8);
				system_sleep();
				waitCounter++;
				if (f_pin == 1) { // Did a pin change wake us up?
					functionSelect = 0;
					
					// Disable the watchdog
					MCUSR &= ~(1<<WDRF);
					WDTCR |= (1<<WDCE) | (1<<WDE);
					WDTCR = 0;
				  
					waitCounter = waitTime; // break out of the loop
					for (int x = 0; x < 2; x++) {
						PORTB |= (1<<ledPin); // ON
						_delay_ms(500);
						PORTB &= ~(1<<ledPin); // OFF
						_delay_ms(500);
					}
				}
			}
		}
		
		// Configure the delay time
		if (functionSelect == 2) {
			int buttonState = PINB & (1<<buttonPin);
			if (buttonState) {
				_delay_ms(500);
				int buttonPressed = false;
				buttonState = PINB & (1<<buttonPin);
				if (buttonState && delayTime >= 0) {
					_delay_ms(1500);
					buttonState = PINB & (1<<buttonPin);
					if (buttonState) {
						functionSelect = 0;
						eeprom_write_byte((uint8_t*)0, delayTime);
						for (int x = 0; x < 3; x++) {
							PORTB |= (1<<ledPin); // ON
							_delay_ms(500);
							PORTB &= ~(1<<ledPin); // OFF
							_delay_ms(500);
						}
					}
					else {
						buttonPressed = true;
					}
				}
				if (buttonState == false || buttonPressed == true) {
					delayTime++;
					PORTB |= (1<<ledPin); // ON
					_delay_ms(500);
					PORTB &= ~(1<<ledPin); // OFF
					
					if (delayTime > 10) { // Too many presses
						delayTime = -1;
						PORTB |= (1<<ledPin); // ON
						_delay_ms(1500);
						PORTB &= ~(1<<ledPin); // OFF
					}
				}
			}
		}
		
		// Send data from EEPROM to Arduino
		if (functionSelect == 3) {
			PORTB |= (1<<ledPin); // ON
			boolean inTransfer = true;
			int readCount = 1;
			while (inTransfer == true) {
				byte dataByte = 0;
				if (readCount < 512) {
					dataByte = eeprom_read_byte((uint8_t*)readCount);
				}
				for (int x = 7; x >= 0; x--) {
					PORTB &= ~(1<<dataPin); // OFF
					
					// Wait until CLK is low
					long resetTimer = millis();
					int clockIn = analogRead(clockPin);
					while (clockIn >= 160 && inTransfer == true) {
						clockIn = analogRead(clockPin);
						if (millis() > resetTimer + 250) {  // Check if host is restarting the CLK
							inTransfer = false;
							functionSelect = 0;
						}          
					}
					
					// Output the bit
					byte dataBit = dataByte & (1<<x);
					if (dataBit) {
						PORTB |= (1<<dataPin);
					}
					else {
						PORTB &= ~(1<<dataPin);
					}
					
					// Wait until CLK is high
					resetTimer = millis();
					clockIn = analogRead(clockPin);
					while (clockIn == 0 && inTransfer == true) {
						clockIn = analogRead(clockPin);
						if (millis() > resetTimer + 100) {
							PORTB &= ~(1<<dataPin); // Don't want to wait 250ms for this otherwise Arduino will think it's beginning from the start again
						}
						if (millis() > resetTimer + 250) { // Check if host is restarting the CLK
							inTransfer = false;
							functionSelect = 0;
						}
					}
				}
				if (dataByte == 0) { // No more data to send
					inTransfer = false;
					functionSelect = 0;
				}
				readCount++;
			}
		}
	}
}

ISR(WDT_vect) {
}

ISR(PCINT0_vect) {
  f_pin = 1; // Set global flag
}